import math
import os.path
import random
import argparse
import seaborn as sns


def get_color_list(color_num):

    if color_num <= 8:
        color_list_combined = ['#3787c0', '#39399f', '#ffb939', '#399f39', '#9f399f', '#fb694a', '#9f9f39', '#959595']
    elif 8 < color_num <= 16:
        color_list_combined = ['#2b7bba', '#89bedc', '#2e2e99', '#8a8acc', '#ffa500', '#ffc55c', '#2e992e', '#8acc8a', '#992e99', '#cc8acc', '#d52221', '#fc8161', '#99992e', '#cccc8a', '#5c5c5c', '#adadad']
    else:
        color_num_each = math.ceil(color_num/8) + 2
        color_list_1 = sns.color_palette('Blues', n_colors=color_num_each).as_hex()
        color_list_2 = sns.light_palette('navy',   n_colors=color_num_each).as_hex()
        color_list_3 = sns.light_palette('orange', n_colors=color_num_each).as_hex()
        color_list_4 = sns.light_palette('green',  n_colors=color_num_each).as_hex()
        color_list_5 = sns.light_palette('purple', n_colors=color_num_each).as_hex()
        color_list_6 = sns.color_palette('Reds',  n_colors=color_num_each).as_hex()
        color_list_7 = sns.light_palette('olive',  n_colors=color_num_each).as_hex()
        color_list_8 = sns.color_palette('Greys', n_colors=color_num_each).as_hex()
        color_list_combined = []
        for color_list in [color_list_1, color_list_2, color_list_3, color_list_4, color_list_5, color_list_6, color_list_7, color_list_8]:
            for color in color_list[2:][::-1]:
                color_list_combined.append(color)

    color_list_to_return = random.sample(color_list_combined, color_num)

    color_list_to_return_sorted = []
    for color_to_return in color_list_combined:
        if color_to_return in color_list_to_return:
            color_list_to_return_sorted.append(color_to_return)

    random.shuffle(color_list_to_return_sorted)

    return color_list_to_return_sorted


def cdb_to_itol_piechart(dereplicated_gnm_id_txt, cdb_tsv, metadata_txt, gnm_id_col, interested_grp_col, group_color_txt, op_txt, op_txt_itol):

    grp_dict = dict()
    col_index = dict()
    for each_gnm in open(metadata_txt):
        each_gnm_split = each_gnm.strip().split('\t')
        if each_gnm.startswith('%s\t' % gnm_id_col):
            col_index = {key: i for i, key in enumerate(each_gnm_split)}
        else:
            gnm_id           = each_gnm_split[col_index[gnm_id_col]]
            interested_grp   = each_gnm_split[col_index[interested_grp_col]]
            grp_dict[gnm_id] = interested_grp

    dereplicated_gnm_set = set()
    for each_gnm in open(dereplicated_gnm_id_txt):
        dereplicated_gnm_set.add(each_gnm.strip().split()[0])

    bin_to_cluster_dict = {}
    cluster_to_bin_dict = {}
    obtained_clusters = set()
    col_index = dict()
    line_num_index = 0
    for each_line in open(cdb_tsv):
        line_num_index += 1
        line_split = each_line.strip().split(',')
        if line_num_index == 1:
            col_index = {key: i for i, key in enumerate(line_split)}
        else:
            gnm_name = line_split[col_index['genome']]
            gnm_id   = '.'.join(gnm_name.split('.')[:-1])
            secondary_cluster = line_split[col_index['secondary_cluster']]
            obtained_clusters.add(secondary_cluster)
            bin_to_cluster_dict[gnm_id] = secondary_cluster
            if secondary_cluster not in cluster_to_bin_dict:
                cluster_to_bin_dict[secondary_cluster] = [gnm_id]
            else:
                cluster_to_bin_dict[secondary_cluster].append(gnm_id)

    op_txt_handle = open(op_txt, 'w')
    op_txt_handle.write('Cluster\tRepresentative\tMembers\n')
    grp_dod = dict()
    all_grp_set = set()
    for each_gnm in dereplicated_gnm_set:
        c_id = bin_to_cluster_dict[each_gnm]
        c_gnm_list_sorted = sorted(list(cluster_to_bin_dict[c_id]))
        grp_list = [grp_dict[i] for i in c_gnm_list_sorted]
        current_grp_dict = dict()
        for grp in grp_list:
            if grp not in current_grp_dict:
                current_grp_dict[grp] = 0
            current_grp_dict[grp] += 1
            all_grp_set.add(grp)
        grp_dod[each_gnm] = current_grp_dict
        op_txt_handle.write('%s\t%s\t%s\n' % (c_id, each_gnm, ','.join(c_gnm_list_sorted)))
    op_txt_handle.close()

    all_grp_list_sorted = sorted(list(all_grp_set))
    grp_color_dict = dict()
    if group_color_txt is not None:
        for each_g in open(group_color_txt):
            each_g_split = each_g.strip().split('\t')
            grp_color_dict[each_g_split[0]] = each_g_split[1]
    else:
        grp_color_list = get_color_list(len(all_grp_list_sorted))
        for (grp, color) in zip(all_grp_list_sorted, grp_color_list):
            grp_color_dict[grp] = color
    grp_color_list = [grp_color_dict.get(i, '#000000') for i in all_grp_list_sorted]

    # write out itol file
    op_txt_itol_handle = open(op_txt_itol, 'w')
    op_txt_itol_handle.write('DATASET_PIECHART\n')
    op_txt_itol_handle.write('SEPARATOR TAB\n')
    op_txt_itol_handle.write('DATASET_LABEL\tPieChart\n')
    op_txt_itol_handle.write('FIELD_COLORS\t%s\n' % '\t'.join([grp_color_dict.get(i, '#000000') for i in all_grp_list_sorted]))
    op_txt_itol_handle.write('FIELD_LABELS\t%s\n' % '\t'.join(all_grp_list_sorted))
    op_txt_itol_handle.write('HEIGHT_FACTOR\t1.8\n')
    op_txt_itol_handle.write('BORDER_WIDTH\t0\n')
    op_txt_itol_handle.write('LEGEND_TITLE\tPieChart\n')
    op_txt_itol_handle.write('LEGEND_SCALE\t1\n')
    op_txt_itol_handle.write('LEGEND_SHAPES\t%s\n' % ('\t'.join(['1']*len(all_grp_list_sorted))))
    op_txt_itol_handle.write('LEGEND_COLORS\t%s\n' % ('\t'.join(grp_color_list)))
    op_txt_itol_handle.write('LEGEND_LABELS\t%s\n' % ('\t'.join(all_grp_list_sorted)))
    op_txt_itol_handle.write('LEGEND_SHAPE_SCALES\t%s\n' % ('\t'.join(['1']*len(all_grp_list_sorted))))
    op_txt_itol_handle.write('\n')
    op_txt_itol_handle.write('\n')
    op_txt_itol_handle.write('DATA\n')
    op_txt_itol_handle.write('#ID\tPosition\tRadius\t%s\n' % ('\t'.join(all_grp_list_sorted)))

    for each_grp in sorted(list(grp_dod.keys())):
        grp_dict = grp_dod[each_grp]
        value_list = []
        for each_g in all_grp_list_sorted:
            g_count = grp_dict.get(each_g, 0)
            value_list.append(g_count)
        value_list_as_str = [str(i) for i in value_list]
        op_txt_itol_handle.write('%s\t-1\t10\t%s\n' % (each_grp, '\t'.join(value_list_as_str)))
    op_txt_itol_handle.close()


dereplicated_gnm_id_txt = '/Users/songweizhi/Desktop/Sponge_r226/03_AOA_genomes_dRep/Cdb_ANI95_dereplicated_genomes.txt'
cdb_tsv                 = '/Users/songweizhi/Desktop/Sponge_r226/03_AOA_genomes_dRep/Cdb_ANI95.csv'
metadata_txt            = '/Users/songweizhi/Desktop/Sponge_r226/AOA_genomes_combined_1221_metadata.txt'
gnm_id_col              = 'Genome'
interested_grp_col      = 'Host_Category'
group_color_txt         = '/Users/songweizhi/Desktop/Sponge_r226/00_Sponge_r220/0_metadata/color_code_habitat.txt'
op_txt                  = '/Users/songweizhi/Desktop/Sponge_r226/03_AOA_genomes_dRep/aaa.csv'
op_txt_itol             = '/Users/songweizhi/Desktop/Sponge_r226/03_AOA_genomes_dRep/aaa_iTOL.csv'


cdb_to_itol_piechart(dereplicated_gnm_id_txt, cdb_tsv, metadata_txt, gnm_id_col, interested_grp_col, group_color_txt, op_txt, op_txt_itol)



import os
import argparse


get_abd1_mask_usage = '''
================================= get_abd1_mask example commands =================================

BioSAK get_abd1_mask -r symbionts_dRep99_sponge_coral_66.fna -o get_abd1_mask_wd

==================================================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def get_abd1_mask(args):

    fasta_file      = args['r']
    op_dir          = args['o']
    force_overwrite = args['f']
    num_threads     = args['t']

    # create op dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output directory detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    # fasta_file = 'symbionts_dRep99_sponge_coral_66.fna'
    _, _, f_base, _ = sep_path_basename_ext(fasta_file)


    # get rRNA regions
    shell_cmd_1_metaxa2_ssu_cmd     = 'metaxa2 --plus --mode m --cpu %s --multi_thread T --table T -g ssu --not_found T -i %s -o %s/%s.metaxa2_ssu'             % (num_threads, fasta_file, op_dir, f_base)
    shell_cmd_2_metaxa2_lsu_cmd     = 'metaxa2 --plus --mode m --cpu %s --multi_thread T --table T -g lsu --not_found T -i %s -o %s/%s.metaxa2_lsu'             % (num_threads, fasta_file, op_dir, f_base)
    shell_cmd_3                     = 'cut -f 1,9,10 %s/%s.metaxa2_ssu.extraction.results > %s/%s.masked_metaxa_ssu.bed'                                        % (op_dir, f_base, op_dir, f_base)
    shell_cmd_4                     = 'cut -f 1,9,10 %s/%s.metaxa2_lsu.extraction.results > %s/%s.masked_metaxa_lsu.bed'                                        % (op_dir, f_base, op_dir, f_base)
    shell_cmd_5                     = 'cat %s/%s.masked_metaxa_ssu.bed %s/%s.masked_metaxa_lsu.bed > %s/%s.masked_metaxa.bed'                                   % (op_dir, f_base, op_dir, f_base, op_dir, f_base)
    shell_cmd_6_barrnap_bac_cmd     = 'barrnap --kingdom bac --threads %s --reject 0.3 %s > %s/%s.barrnap_bac.gff'                                              % (num_threads, fasta_file, op_dir, f_base)
    shell_cmd_7_barrnap_arc_cmd     = 'barrnap --kingdom arc --threads %s --reject 0.3 %s > %s/%s.barrnap_arc.gff'                                              % (num_threads, fasta_file, op_dir, f_base)
    shell_cmd_8_barrnap_euk_cmd     = 'barrnap --kingdom euk --threads %s --reject 0.3 %s > %s/%s.barrnap_euk.gff'                                              % (num_threads, fasta_file, op_dir, f_base)
    shell_cmd_9                     = 'cut -f 1,4,5 %s/%s.barrnap_bac.gff > %s/%s.masked_barrnap_bac.bed'                                                       % (op_dir, f_base, op_dir, f_base)
    shell_cmd_10                    = 'cut -f 1,4,5 %s/%s.barrnap_arc.gff > %s/%s.masked_barrnap_arc.bed'                                                       % (op_dir, f_base, op_dir, f_base)
    shell_cmd_11                    = 'cut -f 1,4,5 %s/%s.barrnap_euk.gff > %s/%s.masked_barrnap_euk.bed'                                                       % (op_dir, f_base, op_dir, f_base)
    shell_cmd_12                    = 'cat %s/%s.masked_barrnap_bac.bed %s/%s.masked_barrnap_arc.bed %s/%s.masked_barrnap_euk.bed > %s/%s.masked_barrnap.bed'   % (op_dir, f_base, op_dir, f_base, op_dir, f_base, op_dir, f_base)
    shell_cmd_13                    = 'cat %s/%s.masked_barrnap.bed %s/%s.masked_metaxa.bed > %s/%s.masked_rrna.bed'                                            % (op_dir, f_base, op_dir, f_base, op_dir, f_base)

    # get tRNA regions
    shell_cmd_14_trna_scan_arc_cmd  = 'tRNAscan-SE -A -b %s/%s.ar.bedformat --thread 36 %s'                                                                     % (op_dir, f_base, fasta_file)
    shell_cmd_15_trna_scan_bac_cmd  = 'tRNAscan-SE -B -b %s/%s.bac.bedformat --thread 36 %s'                                                                    % (op_dir, f_base, fasta_file)
    shell_cmd_16                    = 'cat %s/%s.ar.bedformat %s/%s.bac.bedformat > %s/%s.bedformat'                                                            % (op_dir, f_base, op_dir, f_base, op_dir, f_base)
    shell_cmd_17                    = 'cut -f 1,2,3 %s/%s.bedformat > %s/%s.masked_trnascan.bed'                                                                % (op_dir, f_base, op_dir, f_base)

    # find duplicated areas
    shell_cmd_18                    = 'dustmasker -in %s -out %s/%s.lowcom.out -outfmt acclist'                                                                 % (fasta_file, op_dir, f_base)
    shell_cmd_19                    = "cut -d '>' -f 2 %s/%s.lowcom.out > %s/%s.masked_dustmasker.bed"                                                          % (op_dir, f_base, op_dir, f_base)

    # cover the above area with NNNN
    shell_cmd_20                    = "cat %s/%s.masked_rrna.bed %s/%s.masked_trnascan.bed %s/%s.masked_dustmasker.bed > %s/%s.mask.bed"                        % (op_dir, f_base, op_dir, f_base, op_dir, f_base, op_dir, f_base)
    shell_cmd_21                    = "awk -F'\t' '$2>=0' %s/%s.mask.bed > %s/%s.mask_0.bed"                                                                    % (op_dir, f_base, op_dir, f_base)
    shell_cmd_22                    = "bedtools maskfasta -fi %s -bed %s/%s.mask_0.bed -fo %s/%s.masked.fasta"                                                  % (fasta_file, op_dir, f_base, op_dir, f_base)

    # run cmds
    print(shell_cmd_1_metaxa2_ssu_cmd)
    os.system(shell_cmd_1_metaxa2_ssu_cmd)

    print(shell_cmd_2_metaxa2_lsu_cmd)
    os.system(shell_cmd_2_metaxa2_lsu_cmd)

    print(shell_cmd_3)
    os.system(shell_cmd_3)

    print(shell_cmd_4)
    os.system(shell_cmd_4)

    print(shell_cmd_5)
    os.system(shell_cmd_5)

    print(shell_cmd_6_barrnap_bac_cmd)
    os.system(shell_cmd_6_barrnap_bac_cmd)

    print(shell_cmd_7_barrnap_arc_cmd)
    os.system(shell_cmd_7_barrnap_arc_cmd)

    print(shell_cmd_8_barrnap_euk_cmd)
    os.system(shell_cmd_8_barrnap_euk_cmd)

    print(shell_cmd_9)
    os.system(shell_cmd_9)

    print(shell_cmd_10)
    os.system(shell_cmd_10)

    print(shell_cmd_11)
    os.system(shell_cmd_11)

    print(shell_cmd_12)
    os.system(shell_cmd_12)

    print(shell_cmd_13)
    os.system(shell_cmd_13)

    print(shell_cmd_14_trna_scan_arc_cmd)
    os.system(shell_cmd_14_trna_scan_arc_cmd)

    print(shell_cmd_15_trna_scan_bac_cmd)
    os.system(shell_cmd_15_trna_scan_bac_cmd)

    print(shell_cmd_16)
    os.system(shell_cmd_16)

    print(shell_cmd_17)
    os.system(shell_cmd_17)

    print(shell_cmd_18)
    os.system(shell_cmd_18)

    print(shell_cmd_19)
    os.system(shell_cmd_19)

    print(shell_cmd_20)
    os.system(shell_cmd_20)

    print(shell_cmd_21)
    os.system(shell_cmd_21)

    print(shell_cmd_22)
    os.system(shell_cmd_22)


if __name__ == '__main__':

    abund_parser = argparse.ArgumentParser()
    abund_parser.add_argument('-r', required=True,                          help='reference sequence')
    abund_parser.add_argument('-o', required=True,                          help='output directory')
    abund_parser.add_argument('-t', required=False, type=int, default=1,    help='number of threads, default is 1')
    abund_parser.add_argument('-f', required=False, action="store_true",    help='force overwrite')
    args = vars(abund_parser.parse_args())
    get_abd1_mask(args)



import os
import glob
import argparse
from Bio import SeqIO
import multiprocessing as mp
from BioSAK.global_functions import sep_path_basename_ext


rename_seq_usage = '''
========================= rename_seq example commands =========================

# rename "NODE_941_length_17600_cov_52.7123" to "NODE_941"
BioSAK rename_seq -in Contigs.fa -sep_in "_" -n 2

# rename "Seawater|NODE|941|length|17600|cov|52.7123" to "Seawater_NODE_941"
BioSAK rename_seq -in Contigs.fa -sep_in "|" -sep_out "_" -n 3

# add prefix to all sequences in a fasta file
BioSAK rename_seq -in Contigs.fa -prefix seawater

# rename "NODE_941_length_17600_cov_52.7123" to "Seawater_NODE_941"
BioSAK rename_seq -in Contigs.fa -sep_in "_" -n 2 -prefix Seawater

# rename sequences in multiple files
BioSAK rename_seq -in seq_folder -x fa -sep_in "_" -n 2 -t 12
BioSAK rename_seq -in seq_folder -x fa -prefix prefix_file.txt

# prefix file format: file name (with extension) followed by prefix to be added 
# to the sequences in the file, tab separated.
MAG_1.fa mag1
MAG_2.fa mag2
MAG_3.fa mag3

===============================================================================
'''

def rename_seq_worker(argument_list):

    seq_file_in     = argument_list[0]
    seq_file_out    = argument_list[1]
    sep_in          = argument_list[2]
    sep_out         = argument_list[3]
    column_to_keep  = argument_list[4]
    add_prefix      = argument_list[5]
    one_line        = argument_list[6]

    if sep_out is None:
        sep_out = sep_in

    ctg_file_out_handle = open(seq_file_out, 'w')
    for Seq_record in SeqIO.parse(seq_file_in, 'fasta'):

        if (sep_in is not None) and (add_prefix is None):
            Seq_record_id_new = sep_out.join(Seq_record.id.split(sep_in)[:column_to_keep])

        elif (sep_in is None) and (add_prefix is not None):
            Seq_record_id_new = '%s_%s' % (add_prefix, Seq_record.id)

        elif (sep_in is not None) and (add_prefix is not None):
            Seq_record_id_new = '%s_%s' % (add_prefix, sep_out.join(Seq_record.id.split(sep_in)[:column_to_keep]))

        else:
            Seq_record_id_new = ''
            print('Don\'t know what to do, program exited!')
            exit()

        if one_line is True:
            ctg_file_out_handle.write('>%s\n' % Seq_record_id_new)
            ctg_file_out_handle.write('%s\n' % Seq_record.seq)
        else:
            Seq_record.id = Seq_record_id_new
            SeqIO.write(Seq_record, ctg_file_out_handle, 'fasta')

    ctg_file_out_handle.close()


def rename_seq(args):

    seq_file_in =    args['in']
    file_extension = args['x']
    sep_in =         args['sep_in']
    sep_out =        args['sep_out']
    column_to_keep = args['n']
    add_prefix =     args['prefix']
    one_line =       args['oneline']
    num_threads =    args['t']

    if os.path.isfile(seq_file_in) is True:
        ctg_file_path, ctg_file_basename, ctg_file_ext = sep_path_basename_ext(seq_file_in)
        seq_file_out = '%s/%s_renamed%s' % (ctg_file_path, ctg_file_basename, ctg_file_ext)
        if os.path.isfile(seq_file_out) is True:
            print('Output file detected, program exited: %s' % seq_file_out)
            exit()
        else:
            rename_seq_worker([seq_file_in, seq_file_out, sep_in, sep_out, column_to_keep, add_prefix, one_line])

    if os.path.isdir(seq_file_in) is True:

        if seq_file_in[-1] == '/':
            seq_file_in = seq_file_in[:-1]

        seq_in_folder_no_path = seq_file_in.split('/')[-1]
        seq_out_folder = '%s_renamed' % seq_in_folder_no_path

        seq_in_re   = '%s/*.%s' % (seq_file_in, file_extension)
        seq_in_list = [os.path.basename(file_name) for file_name in glob.glob(seq_in_re)]
        if len(seq_in_list) == 0:
            print('No sequence file detected, program exited!')
            exit()

        if os.path.isdir(seq_out_folder) is True:
            print('Output folder detected, program exited: %s' % seq_out_folder)
            exit()
        else:
            os.mkdir(seq_out_folder)

        prefix_dict = {}
        if add_prefix is not None:
            if os.path.isfile(add_prefix) is False:
                print('Prefix file not detected, program exited!')
                exit()
            else:
                # read in prefix
                for each_genome in open(add_prefix):
                    each_genome_split = each_genome.strip().split('\t')
                    prefix_dict[each_genome_split[0]] = each_genome_split[1]

                genome_without_prefix = set()
                for each_seq_file in seq_in_list:
                    if each_seq_file not in prefix_dict:
                        genome_without_prefix.add(each_seq_file)
                if len(genome_without_prefix) > 0:
                    print('Prefix for the following files not found, , program exited!')
                    print(','.join(genome_without_prefix))
                    exit()

        argument_lol_for_rename_seq_worker = []
        for each_seq_file in seq_in_list:
            pwd_seq_in  = '%s/%s' % (seq_file_in, each_seq_file)
            pwd_seq_out = '%s/%s' % (seq_out_folder, each_seq_file)
            current_argument_list = [pwd_seq_in, pwd_seq_out, sep_in, sep_out, column_to_keep, prefix_dict.get(each_seq_file, None), one_line]
            argument_lol_for_rename_seq_worker.append(current_argument_list)

        # rename sequence files with multiprocessing
        pool = mp.Pool(processes=num_threads)
        pool.map(rename_seq_worker, argument_lol_for_rename_seq_worker)
        pool.close()
        pool.join()

    print('Done!')


if __name__ == '__main__':

    rename_seq_parser = argparse.ArgumentParser()
    rename_seq_parser.add_argument('-in',         required=True,                          help='input sequence file')
    rename_seq_parser.add_argument('-x',          required=False, default='fasta',        help='file extension, default: fasta')
    rename_seq_parser.add_argument('-sep_in',     required=False, default=None,           help='separator for input sequences')
    rename_seq_parser.add_argument('-sep_out',    required=False, default=None,           help='separator for output sequences, default: same as sep_in')
    rename_seq_parser.add_argument('-n',          required=False, default=None, type=int, help='the number of columns to keep')
    rename_seq_parser.add_argument('-prefix',     required=False, default=None,           help='add prefix to sequence')
    rename_seq_parser.add_argument('-oneline',    required=False, action="store_true",    help='put sequence in single line')
    rename_seq_parser.add_argument('-t',          required=False, type=int, default=1,    help='number of threads')

    args = vars(rename_seq_parser.parse_args())
    rename_seq(args)



import os
import glob
import argparse


prefix_file_usage = '''
====================== prefix_file example commands ======================

BioSAK prefix_file -p NS -i refined_MAGs -x fa -o refined_MAGs_renamed

==========================================================================
'''


def sep_path_basename_ext(file_in):
    f_path, file_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(file_name)
    return f_path, f_base, f_ext


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create) is True:
        os.system('rm -r %s' % folder_to_create)
    os.mkdir(folder_to_create)


def prefix_file(args):

    prefix          = args['p']
    dir_in          = args['i']
    file_ext        = args['x']
    dir_out         = args['o']
    force_overwrite = args['f']

    # create op dir
    if force_overwrite is True:
        if os.path.isdir(dir_out) is True:
            os.system('rm -r %s' % dir_out)
        os.system('mkdir %s' % dir_out)
    else:
        if os.path.isdir(dir_out) is False:
            os.system('mkdir %s' % dir_out)

    file_in_re   = '%s/*.%s' % (dir_in, file_ext)
    file_in_list = glob.glob(file_in_re)

    for each_file in file_in_list:
        f_path, f_base, f_ext   = sep_path_basename_ext(each_file)
        pwd_file_renamed        = '%s/%s_%s%s'  % (dir_out, prefix, f_base, f_ext)
        rename_cmd              = 'mv %s %s'    % (each_file, pwd_file_renamed)
        os.system(rename_cmd)


if __name__ == '__main__':

    prefix_file_parser = argparse.ArgumentParser()
    prefix_file_parser.add_argument('-p',   required=True,                          help='add prefix to sequence')
    prefix_file_parser.add_argument('-i',   required=True,                          help='input sequence file')
    prefix_file_parser.add_argument('-x',   required=True,                          help='file extension, default: fasta')
    prefix_file_parser.add_argument('-o',   required=True,                          help='the number of columns to keep')
    prefix_file_parser.add_argument('-f',   required=False, action="store_true",    help='force overwrite')
    args = vars(prefix_file_parser.parse_args())
    prefix_file(args)



import gzip
import sys


def gff2chrom(gff_in, chrom_out):

    # This function was modified based on the NCBIgff2chrom.py from https://github.com/conchoecia/odp
    # This program parses a NCBI GFF annotation and generates a .chrom file
    # see https://github.com/conchoecia/odp for the specification

    gzipped = False
    for thisend in [".gz", ".gzip", ".GZ", ".GZIP", ".gzipped", ".GZIPPED"]:
        if gff_in.endswith(thisend):
            gzipped = True

    if gzipped:
        handle = gzip.open(gff_in,'rt')
    else:
        handle = open(gff_in, "r")

    prots = dict()
    for line in handle:
        line = line.strip()
        splitd=line.split("\t")
        if line and len(splitd) > 7 and splitd[2] == "CDS" and "protein_id=" in line:
            pid = [x for x in splitd[8].split(";") if x.startswith("protein_id=")][0].replace("protein_id=", "")
            scaf = splitd[0]
            strand = splitd[6]
            start = int(splitd[3])
            stop = int(splitd[3])
            if pid not in prots:
                prots[pid] = {"scaf": scaf, "strand": strand, "start": start, "stop": stop}
            else:
                if start < prots[pid]["start"]:
                    prots[pid]["start"] = start
                if stop > prots[pid]["stop"]:
                    prots[pid]["stop"] = stop
    handle.close()

    # write out .chrom file
    chrom_out_handle = open(chrom_out, 'w')
    for pid in prots:
        chrom_out_handle.write("{}\t{}\t{}\t{}\t{}\n".format(pid, prots[pid]["scaf"], prots[pid]["strand"], prots[pid]["start"], prots[pid]["stop"]))
    chrom_out_handle.close()


gff_in    = '/Users/songweizhi/Desktop/Limpet/GCF_932274485.2_xgPatVulg1.2_genomic.gff'
chrom_out = '/Users/songweizhi/Desktop/Limpet/GCF_932274485.2_xgPatVulg1.2_genomic.chrom'
gff2chrom(gff_in, chrom_out)




import os
import glob
import argparse
from Bio import SeqIO


subset_gene_by_gnm_usage = '''
================ subset_gene_by_gnm example commands ================

BioSAK subset_gene_by_gnm -i dir_in -x da -g gnm.txt -o dir_out -f 

=====================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)
    f_ext = f_ext[1:]

    return f_name, f_path, f_base, f_ext


def subset_gene_by_gnm(args):

    file_dir        = args['i']
    file_ext        = args['x']
    gnm_txt         = args['g']
    op_dir          = args['o']
    force_overwrite = args['f']

    # create output folder
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output folder detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    gnm_set = set()
    for each in open(gnm_txt):
        gnm_set.add(each.strip())

    file_re = '%s/*.%s' % (file_dir, file_ext)
    file_list = glob.glob(file_re)

    for each in file_list:
        f_name, f_path, f_base, f_ext = sep_path_basename_ext(each)
        op_fa = '%s/%s' % (op_dir, f_name)
        op_fa_handle = open(op_fa, 'w')
        for each_seq in SeqIO.parse(each, 'fasta'):
            seq_id = each_seq.id
            gnm_id = '_'.join(each_seq.id.split('_')[:-1])
            if gnm_id in gnm_set:
                op_fa_handle.write('>%s\n' % seq_id)
                op_fa_handle.write('%s\n' % str(each_seq.seq))
            else:
                print(seq_id)
        op_fa_handle.close()


if __name__ == '__main__':

    subset_gene_by_gnm_parser = argparse.ArgumentParser(usage=subset_gene_by_gnm_usage)
    subset_gene_by_gnm_parser.add_argument('-i',  required=True,                          help='input folder')
    subset_gene_by_gnm_parser.add_argument('-x',  required=False, default=None,           help='file extension')
    subset_gene_by_gnm_parser.add_argument('-g',  required=False, default=None,           help='genome id')
    subset_gene_by_gnm_parser.add_argument('-o',  required=True,                          help='output folder')
    subset_gene_by_gnm_parser.add_argument('-f',  required=False, action="store_true",    help='force overwrite')
    args = vars(subset_gene_by_gnm_parser.parse_args())
    subset_gene_by_gnm(args)



import argparse

blast_usage = '''
=============================== blast example commands ===============================

BioSAK blast -i Alignment.txt -o Alignment_formatted.txt -n 20
BioSAK blast -i Alignment.txt -o Alignment_formatted.txt -n 20 -iden 90
BioSAK blast -i Alignment.txt -o Alignment_formatted.txt -n 20 -iden 90 -tax tax.txt

# The input file
On the top of the blast result page (the RID row), click "Download All", 
then choose "Text" from the top of the pop-up.

# Use "BioSAK taxdump" module to generate the "-tax" file.
-tax /Users/songweizhi/DB/taxdump_20250321/ncbi_taxonomy.txt

======================================================================================
'''


def blast(args):

    blast_op_txt     = args['i']
    blast_op_txt_fmt = args['o']
    top_hit_num      = args['n']
    iden_cutoff      = args['iden']
    cov_cutoff       = args['cov']
    tax_file         = args['tax']

    blast_op_txt_fmt_id_txt = '.'.join(blast_op_txt_fmt.split('.')[:-1]) + '_ref_accession.txt'
    ref_to_taxon_txt        = '.'.join(blast_op_txt_fmt.split('.')[:-1]) + '_ref_taxon.txt'

    tax_lineage_dict = dict()
    if tax_file is not None:
        for each_line in open(tax_file):
            each_line_split = each_line.strip().split('\t')
            tax_lineage_dict[each_line_split[0]] = each_line_split[1]

    accession_set = set()
    accession_tax_dict = dict()
    current_query = ''
    current_query_len = 0
    keep_line = 0
    wrote_line_num = 0
    blast_op_txt_fmt_handle = open(blast_op_txt_fmt, 'w')
    for each_line in open(blast_op_txt):
        each_line_split = each_line.strip().split()

        # get current_query and current_query_len
        if (each_line.startswith('Query #')) and ('Query ID:' in each_line):
            current_query = each_line.strip().split(' Query ID: ')[0].split(':')[1].strip()
            current_query_len = each_line.strip().split(' Length: ')[1]
            wrote_line_num = 0
            blast_op_txt_fmt_handle.write('\n')

        # decide to keep current line or not
        if each_line_split == ['Description', 'Name', 'Name', 'Taxid', 'Score', 'Score', 'cover', 'Value', 'Ident', 'Len', 'Accession']:
            keep_line = 1
        if len(each_line.strip()) == 0:
            keep_line = 0

        # write out
        if keep_line == 1:
            if wrote_line_num < top_hit_num:
                if each_line_split == ['Description', 'Name', 'Name', 'Taxid', 'Score', 'Score', 'cover', 'Value', 'Ident', 'Len', 'Accession']:
                    blast_op_txt_fmt_handle.write('%s\t%s\t%s\n' % (current_query, each_line.strip(), 'Label'))
                else:
                    iden      = float(each_line_split[-3])
                    cov       = float(each_line_split[-5][:-1])
                    hit_genus = each_line_split[0]
                    accession = each_line_split[-1]
                    hit_tax_lineage= tax_lineage_dict.get(hit_genus, hit_genus)
                    if (iden >= iden_cutoff) and (cov >= cov_cutoff):
                        blast_op_txt_fmt_handle.write('%s\t%s\t%s__%s(%s)\n' % (current_query, each_line.strip(), current_query, hit_tax_lineage, iden))
                        accession_set.add(accession)
                        accession_tax_dict[accession] = hit_tax_lineage
                        wrote_line_num += 1
    blast_op_txt_fmt_handle.close()

    blast_op_txt_fmt_id_txt_handle = open(blast_op_txt_fmt_id_txt, 'w')
    ref_to_taxon_txt_handle = open(ref_to_taxon_txt, 'w')
    for each_accession in sorted(list(accession_set)):
        accession_tax = accession_tax_dict.get(each_accession, 'na')
        blast_op_txt_fmt_id_txt_handle.write('%s\n' % each_accession)
        ref_to_taxon_txt_handle.write('%s\t%s\n' % (each_accession, accession_tax))
    blast_op_txt_fmt_id_txt_handle.close()
    ref_to_taxon_txt_handle.close()

    # # to download the sequence, run:
    # download_seq_cmd = 'cat %s | paste -sd "," - | xargs -I{} esearch -db nucleotide -query {} | efetch -format fasta > %s' % (blast_op_txt_fmt_id_txt, blast_op_txt_fmt_id_txt.replace('.txt', '.fasta'))
    # print(download_seq_cmd)


if __name__ == '__main__':

    blast_parser = argparse.ArgumentParser(usage=blast_usage)
    blast_parser.add_argument('-i',      required=True,                          help='input file')
    blast_parser.add_argument('-o',      required=True,                          help='output file')
    blast_parser.add_argument('-n',      required=False,type=int, default=9999,  help='top hits to keep, default is all')
    blast_parser.add_argument('-iden',   required=False,type=float, default=80,  help='minimum identity, default is 80')
    blast_parser.add_argument('-cov',    required=False,type=float, default=50,  help='minimum coverage, default is 50')
    blast_parser.add_argument('-tax',    required=False, default=None,           help='NCBI taxonomy file')
    args = vars(blast_parser.parse_args())
    blast(args)


'''

cat 28S_ZPVK91YC016-Alignment_reformatted_ref_accession.txt | paste -sd "," - | xargs -I{} esearch -db nucleotide -query {} | efetch -format fasta > 28S_ZPVK91YC016-Alignment_reformatted_ref_accession.fasta


'''


import os
import glob
import math
import argparse
import pandas as pd
from time import sleep
import multiprocessing as mp
from functools import reduce


abd_usage = '''
=========================== abd example commands ===========================

BioSAK abd -i sample.txt -r reference.fa -mask -t 36
BioSAK abd -i sample.txt -r reference_masked.fa -t 36

# input txt file format (tab separated)
Seawater1	path/to/Seawater1_1.fastq.gz	path/to/Seawater1_2.fastq.gz
Seawater2	path/to/Seawater2_1.fastq.gz	path/to/Seawater2_2.fastq.gz

============================================================================
'''


def sep_path_basename_ext(file_in):

    f_path, f_name = os.path.split(file_in)
    if f_path == '':
        f_path = '.'
    f_base, f_ext = os.path.splitext(f_name)

    return f_name, f_path, f_base, f_ext[1:]


def get_abd1_mask(fasta_file, op_dir, num_threads):

    _, _, f_base, _ = sep_path_basename_ext(fasta_file)

    # get rRNA regions
    shell_cmd_1_metaxa2_ssu_cmd     = 'metaxa2 --plus --mode m --cpu %s --multi_thread T --table T -g ssu --not_found T -i %s -o %s/%s.metaxa2_ssu'             % (num_threads, fasta_file, op_dir, f_base)
    shell_cmd_2_metaxa2_lsu_cmd     = 'metaxa2 --plus --mode m --cpu %s --multi_thread T --table T -g lsu --not_found T -i %s -o %s/%s.metaxa2_lsu'             % (num_threads, fasta_file, op_dir, f_base)
    shell_cmd_3                     = 'cut -f 1,9,10 %s/%s.metaxa2_ssu.extraction.results > %s/%s.masked_metaxa_ssu.bed'                                        % (op_dir, f_base, op_dir, f_base)
    shell_cmd_4                     = 'cut -f 1,9,10 %s/%s.metaxa2_lsu.extraction.results > %s/%s.masked_metaxa_lsu.bed'                                        % (op_dir, f_base, op_dir, f_base)
    shell_cmd_5                     = 'cat %s/%s.masked_metaxa_ssu.bed %s/%s.masked_metaxa_lsu.bed > %s/%s.masked_metaxa.bed'                                   % (op_dir, f_base, op_dir, f_base, op_dir, f_base)
    shell_cmd_6_barrnap_bac_cmd     = 'barrnap --kingdom bac --threads %s --reject 0.3 %s > %s/%s.barrnap_bac.gff'                                              % (num_threads, fasta_file, op_dir, f_base)
    shell_cmd_7_barrnap_arc_cmd     = 'barrnap --kingdom arc --threads %s --reject 0.3 %s > %s/%s.barrnap_arc.gff'                                              % (num_threads, fasta_file, op_dir, f_base)
    shell_cmd_8_barrnap_euk_cmd     = 'barrnap --kingdom euk --threads %s --reject 0.3 %s > %s/%s.barrnap_euk.gff'                                              % (num_threads, fasta_file, op_dir, f_base)
    shell_cmd_9                     = 'cut -f 1,4,5 %s/%s.barrnap_bac.gff > %s/%s.masked_barrnap_bac.bed'                                                       % (op_dir, f_base, op_dir, f_base)
    shell_cmd_10                    = 'cut -f 1,4,5 %s/%s.barrnap_arc.gff > %s/%s.masked_barrnap_arc.bed'                                                       % (op_dir, f_base, op_dir, f_base)
    shell_cmd_11                    = 'cut -f 1,4,5 %s/%s.barrnap_euk.gff > %s/%s.masked_barrnap_euk.bed'                                                       % (op_dir, f_base, op_dir, f_base)
    shell_cmd_12                    = 'cat %s/%s.masked_barrnap_bac.bed %s/%s.masked_barrnap_arc.bed %s/%s.masked_barrnap_euk.bed > %s/%s.masked_barrnap.bed'   % (op_dir, f_base, op_dir, f_base, op_dir, f_base, op_dir, f_base)
    shell_cmd_13                    = 'cat %s/%s.masked_barrnap.bed %s/%s.masked_metaxa.bed > %s/%s.masked_rrna.bed'                                            % (op_dir, f_base, op_dir, f_base, op_dir, f_base)

    # get tRNA regions
    shell_cmd_14_trna_scan_arc_cmd  = 'tRNAscan-SE -A -b %s/%s.ar.bedformat --thread 36 %s'                                                                     % (op_dir, f_base, fasta_file)
    shell_cmd_15_trna_scan_bac_cmd  = 'tRNAscan-SE -B -b %s/%s.bac.bedformat --thread 36 %s'                                                                    % (op_dir, f_base, fasta_file)
    shell_cmd_16                    = 'cat %s/%s.ar.bedformat %s/%s.bac.bedformat > %s/%s.bedformat'                                                            % (op_dir, f_base, op_dir, f_base, op_dir, f_base)
    shell_cmd_17                    = 'cut -f 1,2,3 %s/%s.bedformat > %s/%s.masked_trnascan.bed'                                                                % (op_dir, f_base, op_dir, f_base)

    # find duplicated areas
    shell_cmd_18                    = 'dustmasker -in %s -out %s/%s.lowcom.out -outfmt acclist'                                                                 % (fasta_file, op_dir, f_base)
    shell_cmd_19                    = "cut -d '>' -f 2 %s/%s.lowcom.out > %s/%s.masked_dustmasker.bed"                                                          % (op_dir, f_base, op_dir, f_base)

    # cover the above area with NNNN
    shell_cmd_20                    = "cat %s/%s.masked_rrna.bed %s/%s.masked_trnascan.bed %s/%s.masked_dustmasker.bed > %s/%s.mask.bed"                        % (op_dir, f_base, op_dir, f_base, op_dir, f_base, op_dir, f_base)
    shell_cmd_21                    = "awk -F'\t' '$2>=0' %s/%s.mask.bed > %s/%s.mask_0.bed"                                                                    % (op_dir, f_base, op_dir, f_base)
    shell_cmd_22                    = "bedtools maskfasta -fi %s -bed %s/%s.mask_0.bed -fo %s/%s.masked.fasta"                                                  % (fasta_file, op_dir, f_base, op_dir, f_base)

    # run cmds
    print(shell_cmd_1_metaxa2_ssu_cmd)
    os.system(shell_cmd_1_metaxa2_ssu_cmd)

    print(shell_cmd_2_metaxa2_lsu_cmd)
    os.system(shell_cmd_2_metaxa2_lsu_cmd)

    print(shell_cmd_3)
    os.system(shell_cmd_3)

    print(shell_cmd_4)
    os.system(shell_cmd_4)

    print(shell_cmd_5)
    os.system(shell_cmd_5)

    print(shell_cmd_6_barrnap_bac_cmd)
    os.system(shell_cmd_6_barrnap_bac_cmd)

    print(shell_cmd_7_barrnap_arc_cmd)
    os.system(shell_cmd_7_barrnap_arc_cmd)

    print(shell_cmd_8_barrnap_euk_cmd)
    os.system(shell_cmd_8_barrnap_euk_cmd)

    print(shell_cmd_9)
    os.system(shell_cmd_9)

    print(shell_cmd_10)
    os.system(shell_cmd_10)

    print(shell_cmd_11)
    os.system(shell_cmd_11)

    print(shell_cmd_12)
    os.system(shell_cmd_12)

    print(shell_cmd_13)
    os.system(shell_cmd_13)

    print(shell_cmd_14_trna_scan_arc_cmd)
    os.system(shell_cmd_14_trna_scan_arc_cmd)

    print(shell_cmd_15_trna_scan_bac_cmd)
    os.system(shell_cmd_15_trna_scan_bac_cmd)

    print(shell_cmd_16)
    os.system(shell_cmd_16)

    print(shell_cmd_17)
    os.system(shell_cmd_17)

    print(shell_cmd_18)
    os.system(shell_cmd_18)

    print(shell_cmd_19)
    os.system(shell_cmd_19)

    print(shell_cmd_20)
    os.system(shell_cmd_20)

    print(shell_cmd_21)
    os.system(shell_cmd_21)

    print(shell_cmd_22)
    os.system(shell_cmd_22)

    print('Masking reference sequences finished!')


def get_abd2_mapping_worker(arg_list):

    if len(arg_list) == 6:
        op_prefix       = arg_list[0]
        fq_r1           = arg_list[1]
        fq_r2           = arg_list[2]
        ref_seq         = arg_list[3]
        op_dir          = arg_list[4]
        num_threads     = arg_list[5]
    elif len(arg_list) == 5:
        op_prefix       = arg_list[0]
        fq_nonpaired    = arg_list[1]
        ref_seq         = arg_list[2]
        op_dir          = arg_list[3]
        num_threads     = arg_list[4]

    # check input files
    if os.path.isfile(ref_seq) is False:
        print('%s not found, program exited!' % ref_seq)
        exit()

    if len(arg_list) == 6:
        if os.path.isfile(fq_r1) is False:
            print('%s not found, program exited!' % fq_r1)
            exit()
        if os.path.isfile(fq_r2) is False:
            print('%s not found, program exited!' % fq_r2)
            exit()

    if len(arg_list) == 5:
        if os.path.isfile(fq_nonpaired) is False:
            print('%s not found, program exited!' % fq_nonpaired)
            exit()

    os.system('mkdir %s' % op_dir)

    if len(arg_list) == 5:
        fq_nonpaired_name, _, _, _ = sep_path_basename_ext(fq_nonpaired)
    if len(arg_list) == 6:
        fq1_name, _, _, _ = sep_path_basename_ext(fq_r1)
        fq2_name, _, _, _ = sep_path_basename_ext(fq_r2)
    ref_name, _, _, _ = sep_path_basename_ext(ref_seq)

    perform_decompress          = False
    if len(arg_list) == 5:
        fq_nonpaired_decompressed = fq_nonpaired
        if fq_nonpaired.endswith('.gz'):
            fq_nonpaired_decompressed   = '%s/%s'               % (op_dir, fq_nonpaired_name[:-3])
            gunzip_cmd_nonpaired        = 'gunzip -c %s > %s'   % (fq_nonpaired, fq_nonpaired_decompressed)
            perform_decompress          = True
            os.system(gunzip_cmd_nonpaired)

    if len(arg_list) == 6:
        fq_r1_decompressed = fq_r1
        fq_r2_decompressed = fq_r2
        if fq_r1.endswith('.gz'):
            fq_r1_decompressed  = '%s/%s'               % (op_dir, fq1_name[:-3])
            fq_r2_decompressed  = '%s/%s'               % (op_dir, fq2_name[:-3])
            gunzip_cmd_r1       = 'gunzip -c %s > %s'   % (fq_r1, fq_r1_decompressed)
            gunzip_cmd_r2       = 'gunzip -c %s > %s'   % (fq_r2, fq_r2_decompressed)
            perform_decompress  = True
            os.system(gunzip_cmd_r1)
            os.system(gunzip_cmd_r2)

    # index reference sequences
    index_ref_cmd     = 'cp %s %s/; bwa index %s/%s'                                                                                                                        % (ref_seq, op_dir, op_dir, ref_name)

    if len(arg_list) == 6:
        bwa_cmd           = 'bwa mem -5SP -t %s %s/%s %s %s | samblaster > %s/%s.sam'                                                                                       % (num_threads, op_dir, ref_name, fq_r1_decompressed, fq_r2_decompressed, op_dir, op_prefix)
    if len(arg_list) == 5:
        bwa_cmd           = 'bwa mem -5SP -t %s %s/%s %s | samblaster > %s/%s.sam'                                                                                          % (num_threads, op_dir, ref_name, fq_nonpaired_decompressed, op_dir, op_prefix)

    samtools_view_cmd = 'samtools view -@ 32 -bS -h -b %s/%s.sam > %s/%s.bam'                                                                                               % (op_dir, op_prefix, op_dir, op_prefix)
    samtools_sort_cmd = 'samtools sort -@ 32 %s/%s.bam -o %s/%s.sorted.bam'                                                                                                 % (op_dir, op_prefix, op_dir, op_prefix)
    coverm_filter_cmd = 'coverm filter -b %s/%s.sorted.bam --min-read-aligned-percent 0.9 --min-read-percent-identity 0.99 --output-bam-files %s/%s.sorted_filtered.bam'    % (op_dir, op_prefix, op_dir, op_prefix)
    pileup_sh_cmd     = 'pileup.sh in=%s/%s.sorted_filtered.bam out=%s/%s.sorted_filtered.cov rpkm=%s/%s.sorted_filtered.rpkm overwrite=true'                               % (op_dir, op_prefix, op_dir, op_prefix, op_dir, op_prefix)
    if len(arg_list) == 6:
        seqkit_stat_cmd   = 'seqkit stat %s > %s/%s.stat'                                                                                                                       % (fq_r1_decompressed, op_dir, op_prefix)
    if len(arg_list) == 5:
        seqkit_stat_cmd   = 'seqkit stat %s > %s/%s.stat'                                                                                                                       % (fq_nonpaired_decompressed, op_dir, op_prefix)

    print(seqkit_stat_cmd)
    os.system(seqkit_stat_cmd)

    print(index_ref_cmd)
    os.system(index_ref_cmd)

    print(bwa_cmd)
    os.system(bwa_cmd)
    if perform_decompress is True:
        if len(arg_list) == 6:
            os.system('rm %s/%s' % (op_dir, fq1_name[:-3]))
            os.system('rm %s/%s' % (op_dir, fq2_name[:-3]))
        if len(arg_list) == 5:
            os.system('rm %s/%s' % (op_dir, fq_nonpaired_name[:-3]))

    os.system('rm %s/%s'   % (op_dir, ref_name))
    os.system('rm %s/%s.*' % (op_dir, ref_name))

    print(samtools_view_cmd)
    os.system(samtools_view_cmd)
    sleep(1)
    os.system('rm %s/%s.sam' % (op_dir, op_prefix))

    print(samtools_sort_cmd)
    os.system(samtools_sort_cmd)
    sleep(1)
    os.system('rm %s/%s.bam' % (op_dir, op_prefix))

    print(coverm_filter_cmd)
    os.system(coverm_filter_cmd)
    sleep(1)
    os.system('rm %s/%s.sorted.bam' % (op_dir, op_prefix))

    print(pileup_sh_cmd)
    os.system(pileup_sh_cmd)
    sleep(1)
    os.system('rm %s/%s.sorted_filtered.bam' % (op_dir, op_prefix))


def get_abd3_stats(rpkm_stat_cov_dir, stats_dir):

    # check input files
    if os.path.isdir(rpkm_stat_cov_dir) is False:
        print('%s not found, program exited!' % rpkm_stat_cov_dir)
        exit()

    rpkm_file_re    = '%s/*.rpkm' % rpkm_stat_cov_dir
    stat_file_re    = '%s/*.stat' % rpkm_stat_cov_dir
    rpkm_file_list  = glob.glob(rpkm_file_re)
    stat_file_list  = glob.glob(stat_file_re)

    if len(rpkm_file_list) == 0:
        print('.rpkm file not found, program exited!')
        exit()

    if len(stat_file_list) == 0:
        print('.stat file not found, program exited!')
        exit()

    # define output file name
    gnm_level_rpkm_dir       = '%s/rpkm_by_genome'    % stats_dir
    combined_rpkm_file       = '%s/df_rpkm.txt'       % stats_dir
    combined_rpkm_file_log10 = '%s/df_rpkm.log10.txt' % stats_dir

    # create op dir
    os.system('mkdir %s' % stats_dir)
    os.system('mkdir %s' % gnm_level_rpkm_dir)

    # read in stat file
    sample_total_read_num_dict = dict()
    for stat_file in sorted(stat_file_list):
        _, _, stat_base, _ = sep_path_basename_ext(stat_file)
        stat_file_lines = open(stat_file).readlines()
        info_line_split = stat_file_lines[-1].strip().split(' ')
        while '' in info_line_split:
            info_line_split.remove('')
        read_num = int(info_line_split[3].replace(',', ''))
        sample_total_read_num_dict[stat_base] = read_num

    # summarise rpkm on genome level
    sample_id_list = []
    for rpkm_file in sorted(rpkm_file_list):
        rpkm_name, rpkm_path, rpkm_base, rpkm_ext = sep_path_basename_ext(rpkm_file)
        sample_id = rpkm_base.split('.sorted_filtered')[0]
        sample_id_list.append(sample_id)
        op_txt = '%s/%s.cal_bin.rpkm' % (gnm_level_rpkm_dir, sample_id)
        rpkm_df = pd.read_csv(rpkm_file, sep='\t', header=4)
        subset_df_col_list = ['#Name', 'Length', 'Reads']
        sample_total_read_num = sample_total_read_num_dict[sample_id]
        subset_df = rpkm_df.loc[:, subset_df_col_list]
        subset_df['bin_name'] = subset_df['#Name'].str.rsplit('_', n=1, expand=True)[0]
        subset_df['Length'] = subset_df['Length'].astype(int)
        subset_df['Reads'] = subset_df['Reads'].astype(int)

        # calulate RPKM
        op_df = subset_df.groupby(["bin_name"])[['Length', 'Reads']].sum()
        op_df[sample_id] = (op_df['Reads']*1000000000)/(op_df['Length']*sample_total_read_num*2)
        op_df.to_csv(op_txt, sep='\t')

    # combine the results
    rpkm_df_list = []
    for sample_id in sample_id_list:
        gnm_level_rpkm_file = '%s/%s.cal_bin.rpkm' % (gnm_level_rpkm_dir, sample_id)
        gnm_level_rpkm_df = pd.read_csv(gnm_level_rpkm_file, sep='\t')
        gnm_level_rpkm_df = gnm_level_rpkm_df.drop(["Length", "Reads"], axis=1)
        rpkm_df_list.append(gnm_level_rpkm_df)

    # write out combined_rpkm_file
    rpkm_df_combined = reduce(lambda left, right: pd.merge(left, right, on='bin_name',how='outer'), rpkm_df_list)
    rpkm_df_combined.to_csv(combined_rpkm_file, sep='\t',index=False)

    # write out combined_rpkm_file_log10
    combined_rpkm_file_log10_handle = open(combined_rpkm_file_log10, 'w')
    for each_line in open(combined_rpkm_file):
        each_line_split = each_line.strip().split('\t')
        if each_line.startswith('bin_name'):
            combined_rpkm_file_log10_handle.write(each_line)
        else:
            value_list = [each_line_split[0]]
            for each_value in each_line_split[1:]:
                each_value = float(each_value)
                if each_value == 0:
                    value_list.append('na')
                else:
                    each_value_log = math.log(each_value)
                    value_list.append(each_value_log)
            combined_rpkm_file_log10_handle.write('%s\n' % '\t'.join([str(i) for i in value_list]))
    combined_rpkm_file_log10_handle.close()


def abd(args):

    input_txt           = args['i']
    ref_seq             = args['r']
    op_dir              = args['o']
    run_mask            = args['mask']
    non_paired_reads    = args['nonpaired']
    num_threads         = args['t']
    force_overwrite     = args['f']

    # define file name
    mask_wd             = '%s/mask_wd'              % op_dir
    cov_rpkm_stat_dir   = '%s/cov_rpkm_stat_files'  % op_dir
    stats_dir           = '%s/stats_dir'            % op_dir

    # create op dir
    if os.path.isdir(op_dir) is True:
        if force_overwrite is True:
            os.system('rm -r %s' % op_dir)
        else:
            print('Output directory detected, program exited!')
            exit()
    os.system('mkdir %s' % op_dir)

    #################### mask reference sequences ####################

    ref_seq_masked = ref_seq
    if run_mask is True:
        os.system('mkdir %s' % mask_wd)
        get_abd1_mask(ref_seq, mask_wd, num_threads)
        _, _, ref_seq_base, _ = sep_path_basename_ext(ref_seq)
        ref_seq_masked = '%s/%s.masked.fasta' % (mask_wd, ref_seq_base)

    #################### mapping ####################

    # run mapping
    sample_lol = []
    for each_sample in open(input_txt):
        current_arg_list = each_sample.strip().split('\t')
        sample_lol.append(current_arg_list)

    threads_per_job = num_threads//len(sample_lol)

    arg_lol= []
    for arg_list in sample_lol:
        sample_id = arg_list[0]
        current_op_dir = '%s/%s_mapping_wd' % (op_dir, sample_id)
        arg_list.append(ref_seq_masked)
        arg_list.append(current_op_dir)
        arg_list.append(threads_per_job)
        arg_lol.append(arg_list)

    # run mapping with multiprocessing
    pool = mp.Pool(processes=num_threads)
    pool.map(get_abd2_mapping_worker, arg_lol)
    pool.close()
    pool.join()

    #################### get stats ####################

    # mkdir
    os.mkdir(cov_rpkm_stat_dir)
    cp_cmd = 'cp %s/*mapping_wd/* %s' % (op_dir, cov_rpkm_stat_dir)
    os.system(cp_cmd)

    # get stats
    get_abd3_stats(cov_rpkm_stat_dir, stats_dir)

    #################### final report ####################

    # final report
    print('Estimated abundance exported to:')
    print('%s/df_rpkm.txt'       % stats_dir)
    print('%s/df_rpkm.log10.txt' % stats_dir)

    print('Done!')


if __name__ == '__main__':

    get_abd_parser = argparse.ArgumentParser()
    get_abd_parser.add_argument('-i',           required=True,                          help='input txt')
    get_abd_parser.add_argument('-r',           required=True,                          help='reference, need to be masked if you do not specify -mask')
    get_abd_parser.add_argument('-o',           required=True,                          help='output directory')
    get_abd_parser.add_argument('-mask',        required=False, action="store_true",    help='specify to mask reference sequence')
    get_abd_parser.add_argument('-nonpaired',   required=False, action="store_true",    help='specify for nonpaired reads, including long reads')
    get_abd_parser.add_argument('-t',           required=False, type=int, default=1,    help='number of threads, default is 1')
    get_abd_parser.add_argument('-f',           required=False, action="store_true",    help='force overwrite')
    args = vars(get_abd_parser.parse_args())
    abd(args)



import os
import glob
import argparse
from BioSAK.global_functions import force_create_folder


split_folder_parser_usage = '''
================ split_folder example commands ================

BioSAK split_folder -in downloaded_genomes -x fasta -n 10

===============================================================
'''


def split_folder(args):

    file_folder = args['in']
    file_ext    = args['x']
    folder_num  = int(args['n'])

    # create folder
    file_folder_sep = '%s_sep' % file_folder
    force_create_folder(file_folder_sep)

    # get file list
    file_name_re = '%s/*.%s' % (file_folder, file_ext)
    file_name_list = [os.path.basename(file_name) for file_name in glob.glob(file_name_re)]
    file_name_list = sorted(file_name_list)

    # get the number of file per folder
    file_num_per_folder = round(len(file_name_list)/folder_num)

    n = 1
    while n <= folder_num:

        # define current folder name
        current_folder_name = '%s_%s' % (file_folder, n)

        # get file list in current folder
        if n < folder_num:
            files_in_current_folder = file_name_list[(file_num_per_folder * (n - 1)):(file_num_per_folder * n)]
        else:
            files_in_current_folder = file_name_list[(file_num_per_folder * (n - 1)):]

        # create folder
        pwd_current_folder = '%s/%s' % (file_folder_sep, current_folder_name)
        os.system('mkdir %s' % pwd_current_folder)

        # copy files to new folder
        for each_file in files_in_current_folder:
            os.system('cp %s/%s %s/' % (file_folder, each_file, pwd_current_folder))

        n += 1


if __name__ == "__main__":
    parser = argparse.ArgumentParser(usage=split_folder_parser_usage)

    parser.add_argument('-in', required=True, help='file folder')
    parser.add_argument('-x', required=True, help='file extension')
    parser.add_argument('-n', required=False, type=int, help='number of subfolder')

    args = vars(parser.parse_args())

    split_folder(args)



import io
import sys
import pandas as pd


########################################################################################################################

rpkm_file       = '/Users/songweizhi/Desktop/cov_rpkm_stat_files/Carteriospongia1.sorted_filtered.rpkm'
allsample_stat  = '/Users/songweizhi/Desktop/allsample.stat'

'''
python3 /scratch/PI/ocessongwz/Sponge_r220/7_symbiont_abundance/get_abd_scripts/bin_rpkm_calculation.py Cliona1.sorted_filtered.rpkm allsample.stat Cliona1.cal_bin.rpkm
'''

########################################################################################################################

read_num_dict = dict()
for each_line in open('/Users/songweizhi/Desktop/allsample.stat'):
    if not each_line.startswith('file	num_seqs'):
        each_line_split = each_line.strip().split('\t')
        sample_id = each_line_split[0].split('_1.fastq')[0].split('/')[-1]
        read_num = int(each_line_split[1].replace(',', ''))
        read_num_dict[sample_id] = read_num

rpkm_out1 = pd.read_csv(rpkm_file, sep='\t', header=4)

rpkm_out2 = rpkm_out1[['#Name', 'Length', 'Reads']]
rpkm_out2['bin_name'] = rpkm_out2['#Name'].str.rsplit('_', n=1, expand=True)[0]  # contig name 'bin1_1', bin name 'bin1'
rpkm_out2['Length'] = rpkm_out2['Length'].astype(int)
rpkm_out2['Reads'] = rpkm_out2['Reads'].astype(int)

rpkm_out3 = rpkm_out2.groupby(["bin_name"])[['Length', 'Reads']].sum()

sample_name = rpkm_file.split('/')[-1].split('.sorted_filtered.rpkm')[0]
sample_read_num = read_num_dict[sample_name]

# Calulate RPKM
rpkm_out3[sample_name + '_rpkm'] = (rpkm_out3['Reads'] * 1000000000) / (rpkm_out3['Length'] * sample_read_num * 2)
print(rpkm_out3)
